from __future__ import absolute_import, print_function, division, unicode_literals
import unittest
from sklearn.datasets import load_iris, load_boston
from xcessiv import functions
from xcessiv.presets import learnersetting


class TestClassifiers(unittest.TestCase):
    def setUp(self):
        self.X, self.y = load_iris(return_X_y=True)
        self.classifier_settings = [
            'sklearn_random_forest_classifier',
            'sklearn_extra_trees_classifier',
            'sklearn_logistic_regression',
            'sklearn_knn_classifier',
            'sklearn_svm_classifier',
            'sklearn_gaussian_nb',
            'sklearn_adaboost_classifier',
            'xgboost_classifier',
        ]

    def test_learner_settings(self):
        for key in self.classifier_settings:
            setting = getattr(learnersetting, key)
            module = functions.import_string_code_as_module(
                setting['source']
            )

            assert hasattr(module.base_learner, 'get_params')
            assert hasattr(module.base_learner, 'set_params')
            assert hasattr(module.base_learner, 'fit')
            assert hasattr(module.base_learner, setting['meta_feature_generator'])

            module.base_learner.fit(self.X, self.y)

            del module


class TestRegressors(unittest.TestCase):
    def setUp(self):
        self.X, self.y = load_boston(return_X_y=True)
        self.regressor_settings = [
            'sklearn_random_forest_regressor',
            'sklearn_extra_trees_regressor',
            'sklearn_bagging_regressor',
            'sklearn_GP_regressor',
            'sklearn_ridge_regressor',
            'sklearn_lasso_regressor',
            'sklearn_kernel_ridge_regressor',
            'sklearn_knn_regressor',
            'sklearn_svr_regressor',
            'sklearn_decision_tree_regressor',
            'sklearn_linear_regression',
            'sklearn_adaboost_regressor',
            'xgboost_regressor',
        ]

    def test_learner_settings(self):
        for key in self.regressor_settings:
            setting = getattr(learnersetting, key)
            module = functions.import_string_code_as_module(
                setting['source']
            )

            assert hasattr(module.base_learner, 'get_params')
            assert hasattr(module.base_learner, 'set_params')
            assert hasattr(module.base_learner, 'fit')
            assert hasattr(module.base_learner, setting['meta_feature_generator'])

            module.base_learner.fit(self.X, self.y)

            del module


class TestTransformers(unittest.TestCase):
    def setUp(self):
        self.X, self.y = load_boston(return_X_y=True)
        self.transformer_settings = [
            'identity_transformer'
        ]

    def test_learner_settings(self):
        for key in self.transformer_settings:
            setting = getattr(learnersetting, key)
            module = functions.import_string_code_as_module(
                setting['source']
            )

            assert hasattr(module.base_learner, 'get_params')
            assert hasattr(module.base_learner, 'set_params')
            assert hasattr(module.base_learner, 'fit')
            assert hasattr(module.base_learner, setting['meta_feature_generator'])

            module.base_learner.fit(self.X, self.y)

            del module



"""This module contains preset source codes for base learners"""
from __future__ import absolute_import, print_function, division, unicode_literals

__all__ = [

    # Classifiers
    'sklearn_random_forest_classifier_source',
    'sklearn_extra_trees_classifier_source',
    'sklearn_logistic_regression_source',
    'sklearn_knn_classifier_source',
    'sklearn_svm_classifier_source',
    'sklearn_gaussian_nb_source',
    'sklearn_adaboost_classifier_source',
    'xgboost_classifier_source',

    # Regressors
    'sklearn_random_forest_regressor_source',
    'sklearn_extra_trees_regressor_source',
    'sklearn_bagging_regressor_source',
    'sklearn_GP_regressor_source',
    'sklearn_ridge_regressor_source',
    'sklearn_lasso_regressor_source',
    'sklearn_kernel_ridge_regressor_source',
    'sklearn_knn_regressor_source',
    'sklearn_svr_regressor_source',
    'sklearn_decision_tree_regressor_source',
    'sklearn_linear_regression_source',
    'sklearn_adaboost_regressor_source',
    'xgboost_regressor_source',

    # Transformers
    'identity_transformer_source'
]

sklearn_random_forest_classifier_source = \
    """from sklearn.ensemble import RandomForestClassifier

base_learner = RandomForestClassifier(random_state=8)
"""

sklearn_extra_trees_classifier_source = \
    """from sklearn.ensemble import ExtraTreesClassifier

base_learner = ExtraTreesClassifier(random_state=8)
"""

sklearn_logistic_regression_source = \
    """from sklearn.linear_model import LogisticRegression

base_learner = LogisticRegression()
"""

sklearn_knn_classifier_source = \
    """from sklearn.neighbors import KNeighborsClassifier

base_learner = KNeighborsClassifier()
"""

sklearn_svm_classifier_source = \
    """from sklearn.svm import SVC

base_learner = SVC(random_state=8)
"""

sklearn_gaussian_nb_source = \
    """from sklearn.naive_bayes import GaussianNB

base_learner = GaussianNB()
"""

sklearn_adaboost_classifier_source = \
    """from sklearn.ensemble import AdaBoostClassifier

base_learner = AdaBoostClassifier(random_state=8)
"""

xgboost_classifier_source = \
    """from xgboost import XGBClassifier

base_learner = XGBClassifier(seed=8)
"""

sklearn_random_forest_regressor_source = \
    """from sklearn.ensemble import RandomForestRegressor

base_learner = RandomForestRegressor(random_state=8)
"""

sklearn_extra_trees_regressor_source = \
    """from sklearn.ensemble import ExtraTreesRegressor

base_learner = ExtraTreesRegressor(random_state=8)
"""

sklearn_bagging_regressor_source = \
    """from sklearn.ensemble import BaggingRegressor

base_learner = BaggingRegressor(random_state=8)
"""

sklearn_GP_regressor_source = \
    """from sklearn.gaussian_process import GaussianProcessRegressor

base_learner = GaussianProcessRegressor(random_state=8)
"""

sklearn_ridge_regressor_source = \
    """from sklearn.linear_model import Ridge

base_learner = Ridge(random_state=8)
"""

sklearn_lasso_regressor_source = \
    """from sklearn.linear_model import Lasso

base_learner = Lasso(random_state=8)
"""

sklearn_kernel_ridge_regressor_source = \
    """from sklearn.kernel_ridge import KernelRidge

base_learner = KernelRidge()
"""

sklearn_knn_regressor_source = \
    """from sklearn.neighbors import KNeighborsRegressor

base_learner = KNeighborsRegressor()
"""

sklearn_svr_regressor_source = \
    """from sklearn.svm import SVR

base_learner = SVR()
"""

sklearn_decision_tree_regressor_source = \
    """from sklearn.tree import DecisionTreeRegressor

base_learner = DecisionTreeRegressor(random_state=8)
"""

sklearn_linear_regression_source = \
    """from sklearn.linear_model import LinearRegression

base_learner = LinearRegression()
"""

sklearn_adaboost_regressor_source = \
    """from sklearn.ensemble import AdaBoostRegressor

base_learner = AdaBoostRegressor(random_state=8)
"""

xgboost_regressor_source = \
    """from xgboost import XGBRegressor

base_learner = XGBRegressor(seed=8)
"""

identity_transformer_source = \
    """from sklearn.preprocessing import FunctionTransformer

base_learner = FunctionTransformer()
"""



from __future__ import absolute_import, print_function, division, unicode_literals
import unittest
from sklearn.datasets import load_iris
from xcessiv import functions
from xcessiv.presets import cvsetting


class TestKFold(unittest.TestCase):
    def setUp(self):
        self.X, self.y = load_iris(return_X_y=True)

    def test_k_fold_source(self):
        module = functions.import_string_code_as_module(cvsetting.k_fold['source'])
        assert hasattr(module, 'return_splits_iterable')

        list(module.return_splits_iterable(self.X, self.y))

        del module


class TestStratifiedKFold(unittest.TestCase):
    def setUp(self):
        self.X, self.y = load_iris(return_X_y=True)

    def test_source(self):
        module = functions.import_string_code_as_module(cvsetting.stratified_k_fold['source'])
        assert hasattr(module, 'return_splits_iterable')

        list(module.return_splits_iterable(self.X, self.y))

        del module


class TestShuffleSplit(unittest.TestCase):
    def setUp(self):
        self.X, self.y = load_iris(return_X_y=True)

    def test_source(self):
        module = functions.import_string_code_as_module(cvsetting.shuffle_split['source'])
        assert hasattr(module, 'return_splits_iterable')

        list(module.return_splits_iterable(self.X, self.y))

        del module


class TestStratifiedShuffleSplit(unittest.TestCase):
    def setUp(self):
        self.X, self.y = load_iris(return_X_y=True)

    def test_source(self):
        module = functions.import_string_code_as_module(cvsetting.stratified_shuffle_split['source'])
        assert hasattr(module, 'return_splits_iterable')

        list(module.return_splits_iterable(self.X, self.y))

        del module


class TestLeaveOneOut(unittest.TestCase):
    def setUp(self):
        self.X, self.y = load_iris(return_X_y=True)

    def test_source(self):
        module = functions.import_string_code_as_module(cvsetting.leave_one_out['source'])
        assert hasattr(module, 'return_splits_iterable')

        list(module.return_splits_iterable(self.X, self.y))

        del module


class TestLeavePOut(unittest.TestCase):
    def setUp(self):
        self.X, self.y = load_iris(return_X_y=True)

    def test_source(self):
        module = functions.import_string_code_as_module(cvsetting.leave_p_out['source'])
        assert hasattr(module, 'return_splits_iterable')

        list(module.return_splits_iterable(self.X, self.y))

        del module


class TestGroupKFold(unittest.TestCase):
    def setUp(self):
        self.X, self.y = load_iris(return_X_y=True)

    def test_source(self):
        module = functions.import_string_code_as_module(cvsetting.group_k_fold['source'])
        assert hasattr(module, 'return_splits_iterable')

        generator = module.return_splits_iterable(self.X, self.y)
        self.assertRaises(
            ValueError,
            list,
            generator
        )

        del module


class TestTimeSeriesSplit(unittest.TestCase):
    def setUp(self):
        self.X, self.y = load_iris(return_X_y=True)

    def test_source(self):
        module = functions.import_string_code_as_module(cvsetting.leave_one_out['source'])
        assert hasattr(module, 'return_splits_iterable')

        list(module.return_splits_iterable(self.X, self.y))

        del module



"""This module contains preset settings for defining base learners"""
from __future__ import absolute_import, print_function, division, unicode_literals
from xcessiv.presets import learnersource

__all__ = [

    # Classifiers
    'sklearn_random_forest_classifier',
    'sklearn_extra_trees_classifier',
    'sklearn_logistic_regression',
    'sklearn_knn_classifier',
    'sklearn_svm_classifier',
    'sklearn_gaussian_nb',
    'sklearn_adaboost_classifier',
    'xgboost_classifier',

    # Regressors
    'sklearn_random_forest_regressor',
    'sklearn_extra_trees_regressor',
    'sklearn_bagging_regressor',
    'sklearn_GP_regressor',
    'sklearn_ridge_regressor',
    'sklearn_lasso_regressor',
    'sklearn_kernel_ridge_regressor',
    'sklearn_knn_regressor',
    'sklearn_svr_regressor',
    'sklearn_decision_tree_regressor',
    'sklearn_linear_regression',
    'sklearn_adaboost_regressor',
    'xgboost_regressor',

    # Transformers
    'identity_transformer'
]

# Classifiers
sklearn_random_forest_classifier = {
    'name': 'scikit-learn Random Forest Classifier',
    'source': learnersource.sklearn_random_forest_classifier_source,
    'meta_feature_generator': 'predict_proba'
}

sklearn_extra_trees_classifier = {
    'name': 'scikit-learn Extra Trees Classifier',
    'source': learnersource.sklearn_extra_trees_classifier_source,
    'meta_feature_generator': 'predict_proba'
}

sklearn_logistic_regression = {
    'name': 'scikit-learn Logistic Regression',
    'source': learnersource.sklearn_logistic_regression_source,
    'meta_feature_generator': 'predict_proba'
}

sklearn_knn_classifier = {
    'name': 'scikit-learn KNN Classifier',
    'source': learnersource.sklearn_knn_classifier_source,
    'meta_feature_generator': 'predict_proba'
}

sklearn_svm_classifier = {
    'name': 'scikit-learn SVM Classifier',
    'source': learnersource.sklearn_svm_classifier_source,
    'meta_feature_generator': 'decision_function'
}

sklearn_gaussian_nb = {
    'name': 'scikit-learn Gaussian Naive Bayes',
    'source': learnersource.sklearn_gaussian_nb_source,
    'meta_feature_generator': 'predict_proba'
}

sklearn_adaboost_classifier = {
    'name': 'scikit-learn AdaBoost Classifier',
    'source': learnersource.sklearn_adaboost_classifier_source,
    'meta_feature_generator': 'predict_proba'
}

xgboost_classifier = {
    'name': 'XGBoost Classifier',
    'source': learnersource.xgboost_classifier_source,
    'meta_feature_generator': 'predict_proba'
}

# Regressors
sklearn_random_forest_regressor = {
    'name': 'scikit-learn Random Forest Regressor',
    'source': learnersource.sklearn_random_forest_regressor_source,
    'meta_feature_generator': 'predict'
}

sklearn_extra_trees_regressor = {
    'name': 'scikit-learn Extra Trees Regressor',
    'source': learnersource.sklearn_extra_trees_regressor_source,
    'meta_feature_generator': 'predict'
}

sklearn_bagging_regressor = {
    'name': 'scikit-learn Bagging Regressor',
    'source': learnersource.sklearn_bagging_regressor_source,
    'meta_feature_generator': 'predict'
}

sklearn_GP_regressor = {
    'name': 'scikit-learn Gaussian Process Regressor',
    'source': learnersource.sklearn_GP_regressor_source,
    'meta_feature_generator': 'predict'
}

sklearn_ridge_regressor = {
    'name': 'scikit-learn Ridge Regressor',
    'source': learnersource.sklearn_ridge_regressor_source,
    'meta_feature_generator': 'predict'
}

sklearn_lasso_regressor = {
    'name': 'scikit-learn Lasso Regressor',
    'source': learnersource.sklearn_lasso_regressor_source,
    'meta_feature_generator': 'predict'
}

sklearn_kernel_ridge_regressor = {
    'name': 'scikit-learn Kernel Ridge Regressor',
    'source': learnersource.sklearn_kernel_ridge_regressor_source,
    'meta_feature_generator': 'predict'
}

sklearn_knn_regressor = {
    'name': 'scikit-learn K-NN Regressor',
    'source': learnersource.sklearn_knn_regressor_source,
    'meta_feature_generator': 'predict'
}

sklearn_svr_regressor = {
    'name': 'scikit-learn Support Vector Regressor',
    'source': learnersource.sklearn_svr_regressor_source,
    'meta_feature_generator': 'predict'
}

sklearn_decision_tree_regressor = {
    'name': 'scikit-learn Decision Tree Regressor',
    'source': learnersource.sklearn_decision_tree_regressor_source,
    'meta_feature_generator': 'predict'
}

sklearn_linear_regression = {
    'name': 'scikit-learn Linear Regression',
    'source': learnersource.sklearn_linear_regression_source,
    'meta_feature_generator': 'predict'
}

sklearn_adaboost_regressor = {
    'name': 'scikit-learn AdaBoost Regressor',
    'source': learnersource.sklearn_adaboost_regressor_source,
    'meta_feature_generator': 'predict'
}

xgboost_regressor = {
    'name': 'XGBoost Regressor',
    'source': learnersource.xgboost_regressor_source,
    'meta_feature_generator': 'predict'
}


identity_transformer = {
    'name': 'Identity Transformer',
    'source': learnersource.identity_transformer_source,
    'meta_feature_generator': 'transform'
}
